@@ -32,10 +32,12 @@ func TestReadLinesOffsetN(t *testing.T) {
 
 func TestIntToString(t *testing.T) {
 	src := []int8{65, 66, 67}
-	if dst := IntToString(src); dst != "ABC" {
+	dst := IntToString(src)
+	if dst != "ABC" {
 		t.Error("could not convert")
 	}
 }
+
 func TestByteToString(t *testing.T) {
 	src := []byte{65, 66, 67}
 	dst := ByteToString(src)
@@ -57,16 +59,19 @@ func TestHexToUint32(t *testing.T) {
 }
 
 func TestMustParseInt32(t *testing.T) {
-	if ret := mustParseInt32("11111"); ret != int32(11111) {
+	ret := mustParseInt32("11111")
+	if ret != int32(11111) {
 		t.Error("could not parse")
 	}
 }
+
 func TestMustParseUint64(t *testing.T) {
 	ret := mustParseUint64("11111")
 	if ret != uint64(11111) {
 		t.Error("could not parse")
 	}
 }
+
 func TestMustParseFloat64(t *testing.T) {
 	ret := mustParseFloat64("11111.11")
 	if ret != float64(11111.11) {
@@ -77,6 +82,7 @@ func TestMustParseFloat64(t *testing.T) {
 		t.Error("could not parse")
 	}
 }
+
 func TestStringsContains(t *testing.T) {
 	target, err := ReadLines("common_test.go")
 	if err != nil {
@@ -100,7 +106,8 @@ func TestHostEtc(t *testing.T) {
 	if runtime.GOOS == "windows" {
 		t.Skip("windows doesn't have etc")
 	}
-	if p := HostEtc("mtab"); p != "/etc/mtab" {
+	p := HostEtc("mtab")
+	if p != "/etc/mtab" {
 		t.Errorf("invalid HostEtc, %s", p)
 	}
 }
