@@ -1,6 +1,7 @@
 package process
 
 import (
+	"errors"
 	"fmt"
 	"io/ioutil"
 	"net"
@@ -16,14 +17,14 @@ import (
 	"testing"
 	"time"
 
-	"github.com/shirou/gopsutil/internal/common"
+	"github.com/shirou/gopsutil/v3/internal/common"
 	"github.com/stretchr/testify/assert"
 )
 
 var mu sync.Mutex
 
 func skipIfNotImplementedErr(t *testing.T, err error) {
-	if err == common.ErrNotImplementedError {
+	if errors.Is(err, common.ErrNotImplementedError) {
 		t.Skip("not implemented")
 	}
 }
@@ -45,25 +46,6 @@ func Test_Pids(t *testing.T) {
 	}
 }
 
-func Test_Pids_Fail(t *testing.T) {
-	if runtime.GOOS != "darwin" {
-		t.Skip("darwin only")
-	}
-
-	mu.Lock()
-	defer mu.Unlock()
-
-	invoke = common.FakeInvoke{Suffix: "fail"}
-	ret, err := Pids()
-	skipIfNotImplementedErr(t, err)
-	invoke = common.Invoke{}
-	if err != nil {
-		t.Errorf("error %v", err)
-	}
-	if len(ret) != 9 {
-		t.Errorf("wrong getted pid nums: %v/%d", ret, len(ret))
-	}
-}
 func Test_Pid_exists(t *testing.T) {
 	checkPid := os.Getpid()
 
@@ -92,7 +74,6 @@ func Test_NewProcess(t *testing.T) {
 			t.Errorf("error %v", ret)
 		}
 	}
-
 }
 
 func Test_Process_memory_maps(t *testing.T) {
@@ -130,6 +111,7 @@ func Test_Process_memory_maps(t *testing.T) {
 		t.Errorf("memory map is empty")
 	}
 }
+
 func Test_Process_MemoryInfo(t *testing.T) {
 	p := testGetProcess()
 
@@ -195,8 +177,11 @@ func Test_Process_Status(t *testing.T) {
 	if err != nil {
 		t.Errorf("getting status error %v", err)
 	}
-	if v != "R" && v != "S" {
-		t.Errorf("could not get state %v", v)
+	if len(v) == 0 {
+		t.Errorf("could not get state")
+	}
+	if v[0] != Running && v[0] != Sleep {
+		t.Errorf("got wrong state, %v", v)
 	}
 }
 
@@ -358,6 +343,7 @@ func Test_Process_Long_Name_With_Spaces(t *testing.T) {
 	}
 	cmd.Process.Kill()
 }
+
 func Test_Process_Long_Name(t *testing.T) {
 	tmpdir, err := ioutil.TempDir("", "")
 	if err != nil {
@@ -403,6 +389,7 @@ func Test_Process_Long_Name(t *testing.T) {
 	}
 	cmd.Process.Kill()
 }
+
 func Test_Process_Exe(t *testing.T) {
 	p := testGetProcess()
 
@@ -657,6 +644,13 @@ func Test_CPUTimes(t *testing.T) {
 }
 
 func Test_OpenFiles(t *testing.T) {
+	fp, err := os.Open("process_test.go")
+	assert.Nil(t, err)
+	defer func() {
+		err := fp.Close()
+		assert.Nil(t, err)
+	}()
+
 	pid := os.Getpid()
 	p, err := NewProcess(int32(pid))
 	skipIfNotImplementedErr(t, err)
@@ -720,22 +714,6 @@ func Test_IsRunning(t *testing.T) {
 	}
 }
 
-func Test_Process_Cwd(t *testing.T) {
-	myPid := os.Getpid()
-	currentWorkingDirectory, _ := os.Getwd()
-
-	process, _ := NewProcess(int32(myPid))
-	pidCwd, err := process.Cwd()
-	skipIfNotImplementedErr(t, err)
-	if err != nil {
-		t.Fatalf("getting cwd error %v", err)
-	}
-	pidCwd = strings.TrimSuffix(pidCwd, string(os.PathSeparator))
-	assert.Equal(t, currentWorkingDirectory, pidCwd)
-
-	t.Log(pidCwd)
-}
-
 func Test_Process_Environ(t *testing.T) {
 	tmpdir, err := ioutil.TempDir("", "")
 	if err != nil {
@@ -790,6 +768,22 @@ func Test_Process_Environ(t *testing.T) {
 	}
 }
 
+func Test_Process_Cwd(t *testing.T) {
+	myPid := os.Getpid()
+	currentWorkingDirectory, _ := os.Getwd()
+
+	process, _ := NewProcess(int32(myPid))
+	pidCwd, err := process.Cwd()
+	skipIfNotImplementedErr(t, err)
+	if err != nil {
+		t.Fatalf("getting cwd error %v", err)
+	}
+	pidCwd = strings.TrimSuffix(pidCwd, string(os.PathSeparator))
+	assert.Equal(t, currentWorkingDirectory, pidCwd)
+
+	t.Log(pidCwd)
+}
+
 func Test_AllProcesses_cmdLine(t *testing.T) {
 	procs, err := Processes()
 	skipIfNotImplementedErr(t, err)
